package eu.beesoft.gantt.demo;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import eu.beesoft.gaia.swing.JRolloverButton;
import eu.beesoft.gaia.swing.JTreeTable;
import eu.beesoft.gantt.GanttModel;
import eu.beesoft.gantt.JGantt;
import eu.beesoft.gantt.action.CollapseAllNodesAction;
import eu.beesoft.gantt.action.CreateNodeAction;
import eu.beesoft.gantt.action.DeleteNodeAction;
import eu.beesoft.gantt.action.ExpandAllNodesAction;
import eu.beesoft.gantt.action.IndentNodeAction;
import eu.beesoft.gantt.action.MoveNodeDownAction;
import eu.beesoft.gantt.action.MoveNodeUpAction;
import eu.beesoft.gantt.action.RedoAction;
import eu.beesoft.gantt.action.TodayAction;
import eu.beesoft.gantt.action.UndoAction;
import eu.beesoft.gantt.action.UnindentNodeAction;
import eu.beesoft.gantt.action.ZoomInAction;
import eu.beesoft.gantt.action.ZoomOutAction;
import eu.beesoft.gantt.chart.BackgroundLayer;
import eu.beesoft.gantt.chart.CalendarLayer;
import eu.beesoft.gantt.chart.DependencyLayer;
import eu.beesoft.gantt.chart.GanttNodeLayer;
import eu.beesoft.gantt.chart.GridLayer;
import eu.beesoft.gantt.chart.LabelLayer;
import eu.beesoft.gantt.chart.TodayLayer;
import eu.beesoft.gantt.treetable.GanttColumn;
import eu.beesoft.gantt.treetable.GanttNode;

@SuppressWarnings ("serial")
public class DemoGantt extends JGantt {

	private static int nextTaskId;

	private static Action createNodeAction;

	private static Action editNodeAction;

	private static Action deleteNodeAction;

	private static Action indentNodeAction;

	private static Action unindentNodeAction;

	private static Action moveNodeUpAction;

	private static Action moveNodeDownAction;

	private static Action expandAllNodesAction;

	private static Action collapseAllNodesAction;

	private static Action zoomInAction;

	private static Action zoomOutAction;

	private static Action todayAction;

	private static Action undoAction;

	private static Action redoAction;

	private List<Action> popupActions;



	public DemoGantt (GanttModel model) {
		super (model);
		// setResourceBundleName ("eu.beesoft.gantt.JGantt");
	}



	@Override
	protected List<Action> getPopupActions (List<GanttNode> selectedNodes) {
		if (popupActions == null) {
			popupActions = new ArrayList<Action> ();
			popupActions.add (createNodeAction);
			popupActions.add (editNodeAction);
			popupActions.add (deleteNodeAction);
			popupActions.add (null);
			popupActions.add (indentNodeAction);
			popupActions.add (unindentNodeAction);
			popupActions.add (moveNodeUpAction);
			popupActions.add (moveNodeDownAction);
		}
		return popupActions;
	}



	@Override
	public void doubleClicked (GanttNode node) {
		Task task = (Task) node.getUserObject ();
		TaskPropertiesAction.showTaskProperties (task, this);
	}



	public static void main (String[] arg) {

		// create model
		// it extends GanttModel
		GanttModel model = new Model ();

		// create domain objects
		// they should come from database or server in real application
		Resource worker_1 = new Resource ();
		worker_1.setName ("Smith John");
		Resource worker_2 = new Resource ();
		worker_2.setName ("Gates Steven");
		Resource worker_3 = new Resource ();
		worker_3.setName ("Krugman George");
		Resource worker_4 = new Resource ();
		worker_4.setName ("Dewan Annie");

		Task task_1_1 = createTask ("Task 1.1", -20, +1, 92);
		task_1_1.setAssignee (worker_1);
		Task task_1_2 = createTask ("Task 1.2", -14, +2, 42);
		task_1_2.setAssignee (worker_2);
		Task task_1_3 = createTask ("Task 1.3", +2, +4, 16);
		task_1_3.setAssignee (worker_3);
		Task task_1 = createTask ("Task 1", 0, 0, 0);
		task_1.addSubtask (task_1_1);
		task_1.addSubtask (task_1_2);
		task_1.addSubtask (task_1_3);

		Task task_2_1 = createTask ("Task 2.1", +5, +7, 22);
		task_2_1.setAssignee (worker_4);
		Task task_2_2 = createTask ("Task 2.2", +6, +9, 24);
		Task task_2_3 = createTask ("Task 2.3", +8, +10, 28);
		Task task_2 = createTask ("Task 2", 0, 0, 0);
		task_2.addSubtask (task_2_1);
		task_2.addSubtask (task_2_2);
		task_2.addSubtask (task_2_3);

		TaskDependency dependency = new TaskDependency (task_2, task_1);

		Task task_3_1 = createTask ("Task 3.1", -2, +8, 48);
		Task task_3_2 = createTask ("Task 3.2", +4, +6, 16);
		Task task_3_3 = createTask ("Task 3.3", +12, +18, 32);
		Task task_3 = createTask ("Task 3", 0, 0, 0);
		task_3.addSubtask (task_3_1);
		task_3.addSubtask (task_3_2);
		task_3.addSubtask (task_3_3);

		Task task_4_1 = createTask ("Task 4.1", +3, +8, 24);
		Task task_4_2 = createTask ("Task 4.2", +6, +9, 16);
		Task task_4_3 = createTask ("Task 4.3", +12, +16, 22);
		Task task_4 = createTask ("Task 4", 0, 0, 0);
		task_4.addSubtask (task_4_1);
		task_4.addSubtask (task_4_2);
		task_4.addSubtask (task_4_3);

		Project project = new Project ();
		project.setName ("Project");
		project.addSubtask (task_1);
		project.addSubtask (task_2);
		project.addSubtask (task_3);
		project.addSubtask (task_4);
		project.addResource (worker_1);
		project.addResource (worker_2);
		project.addResource (worker_3);
		project.addResource (worker_4);

		// set created project as gantt model root object
		model.setRootObject (project);

		// set default look and feel
		try {
			String plaf = UIManager.getSystemLookAndFeelClassName ();
			UIManager.setLookAndFeel (plaf);
		}
		catch (Exception e) {
			// do nothing
		}

		// create gantt component with "zebra" background
		// and vertical column-line-separators
		JGantt gantt = new DemoGantt (model);
		JTreeTable treeTable = gantt.getTreeTable ();
		treeTable.setAlternateBackground (treeTable.getDefaultAlternateBackground ());
		treeTable.setShowVerticalLines (true);

		// create columns in gantt tree-table
		// this is the first column with task tree
		GanttColumn column = new GanttColumn ();
		column.setHeaderValue ("Task name");
		column.setBinding ("name");
		gantt.addColumn (column);

		column = new GanttColumn ();
		column.setHeaderValue ("Task ID");
		column.setBinding ("taskId");
		gantt.addColumn (column);

		column = new GanttColumn ();
		column.setHeaderValue ("Start date");
		column.setBinding ("startDate");
		column.setCellEditor (new DateCellEditor ());
		gantt.addColumn (column);

		column = new GanttColumn ();
		column.setHeaderValue ("Finish date");
		column.setBinding ("endDate");
		column.setCellEditor (new DateCellEditor ());
		gantt.addColumn (column);

		// resize columns to appropriate width
		gantt.getTreeTable ().pack ();

		// create layers for gantt chart
		// order of layers is important for painting
		gantt.addLayer (new BackgroundLayer ());
		gantt.addLayer (new CalendarLayer ());
		gantt.addLayer (new GridLayer ());
		gantt.addLayer (new TodayLayer ());
		gantt.addLayer (new GanttNodeLayer ());
		gantt.addLayer (new LabelLayer ());
		gantt.addLayer (new DependencyLayer ());

		// create gantt actions
		createNodeAction = new CreateNodeAction (gantt);
		editNodeAction = new TaskPropertiesAction (gantt);
		deleteNodeAction = new DeleteNodeAction (gantt);
		indentNodeAction = new IndentNodeAction (gantt);
		unindentNodeAction = new UnindentNodeAction (gantt);
		moveNodeUpAction = new MoveNodeUpAction (gantt);
		moveNodeDownAction = new MoveNodeDownAction (gantt);
		expandAllNodesAction = new ExpandAllNodesAction (gantt);
		collapseAllNodesAction = new CollapseAllNodesAction (gantt);
		zoomInAction = new ZoomInAction (gantt);
		zoomOutAction = new ZoomOutAction (gantt);
		todayAction = new TodayAction (gantt);
		undoAction = new UndoAction (gantt);
		redoAction = new RedoAction (gantt);

		// build menu
		// TODO
		JMenuBar menuBar = new JMenuBar ();
		JMenu menu = addMenu ("File", menuBar);
		menu = addMenu ("Edit", menuBar);
		menu = addMenu ("Task", menuBar);
		menu = addMenu ("Resource", menuBar);
		menu = addMenu ("Help", menuBar);

		// build toolbar
		JToolBar toolBar = new JToolBar ();
		toolBar.setRollover (true);
		addToToolbar (createNodeAction, toolBar);
		addToToolbar (editNodeAction, toolBar);
		addToToolbar (deleteNodeAction, toolBar);
		toolBar.addSeparator ();
		addToToolbar (unindentNodeAction, toolBar);
		addToToolbar (moveNodeUpAction, toolBar);
		addToToolbar (moveNodeDownAction, toolBar);
		addToToolbar (indentNodeAction, toolBar);
		toolBar.addSeparator ();
		addToToolbar (undoAction, toolBar);
		addToToolbar (redoAction, toolBar);
		toolBar.addSeparator ();
		addToToolbar (expandAllNodesAction, toolBar);
		addToToolbar (collapseAllNodesAction, toolBar);
		toolBar.addSeparator ();
		addToToolbar (zoomInAction, toolBar);
		addToToolbar (zoomOutAction, toolBar);
		addToToolbar (todayAction, toolBar);

		// show gantt diagram in a frame
		JFrame frame = new JFrame ("JGantt demo");
		frame.setDefaultCloseOperation (WindowConstants.EXIT_ON_CLOSE);
		frame.setJMenuBar (menuBar);
		frame.getContentPane ().add (toolBar, BorderLayout.NORTH);
		frame.getContentPane ().add (gantt);
		frame.setSize (1000, 700);
		frame.setVisible (true);
	}



	private static JMenu addMenu (String name, JComponent parent) {
		JMenu menu = new JMenu (name);
		parent.add (menu);
		return menu;
	}



	private static JMenuItem addMenuItem (Action action, JMenu menu) {
		JMenuItem menuItem = new JMenuItem (action);
		menu.add (menuItem);
		return menuItem;
	}



	private static void addToToolbar (Action action, JToolBar toolbar) {
		JRolloverButton button = new JRolloverButton (action);
		if (button.getIcon () != null) {
			button.setText (null);
		}
		toolbar.add (button);
	}



	private static Task createTask (String name, int startDayOffset, int endDayOffset, int effort) {
		Task task = new Task ();
		task.setName (name);
		task.setTaskId (getNextTaskId ());
		if (startDayOffset != 0) {
			Date start = toDate (startDayOffset);
			task.setStartDate (start);
		}
		if (endDayOffset != 0) {
			Date end = toDate (endDayOffset);
			task.setEndDate (end);
		}
		task.setEffort (effort);
		return task;
	}



	private static Date toDate (int daysFromToday) {
		Calendar calendar = Calendar.getInstance ();
		calendar.setTimeInMillis (System.currentTimeMillis ());
		calendar.add (Calendar.DAY_OF_MONTH, daysFromToday);
		return calendar.getTime ();
	}



	public static String getNextTaskId () {
		int id = nextTaskId;
		++nextTaskId;
		return Integer.toString (id);
	}
}
